package com.jenny.networked_chests.items;

import net.minecraft.ChatFormatting;
import net.minecraft.core.BlockPos;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.ChestBlockEntity;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

import static com.jenny.networked_chests.Networked_chests.MODID;

public class ChestInterface extends Item {
    public ChestInterface(Properties properties) {
        super(properties);
    }

    @Override
    public @NotNull InteractionResult useOn(UseOnContext pContext) {
        if (!pContext.getLevel().isClientSide) {
            if (isChest(pContext)) {
                setChest(pContext);
            } else {
                openChest(pContext.getLevel(), (ServerPlayer) pContext.getPlayer(), pContext.getItemInHand());
            }
        }
        return InteractionResult.SUCCESS;
    }

    @Override
    public @NotNull InteractionResultHolder<ItemStack> use(@NotNull Level pLevel, @NotNull Player pPlayer, @NotNull InteractionHand pUsedHand) {
        if (!pLevel.isClientSide) {
            openChest(pLevel, (ServerPlayer) pPlayer, pPlayer.getItemInHand(pUsedHand));
        }
        return InteractionResultHolder.success(pPlayer.getItemInHand(pUsedHand));
    }

    private boolean isChest(@NotNull UseOnContext pContext) {
        return pContext.getLevel().getBlockState(pContext.getClickedPos()).getBlock().getDescriptionId().equals("block.minecraft.chest");
    }

    private boolean isChest(Level level, BlockPos blockPos) {
        return level.getBlockState(blockPos).getBlock().getDescriptionId().equals("block.minecraft.chest");
    }

    private void setChest(@NotNull UseOnContext pContext) {
        BlockPos blockPos = pContext.getClickedPos();
        CompoundTag chestLocation = pContext.getItemInHand().getTag();
        if (chestLocation == null) {
            chestLocation = new CompoundTag();
        }
        chestLocation.putInt("chest_pos_x", blockPos.getX());
        chestLocation.putInt("chest_pos_y", blockPos.getY());
        chestLocation.putInt("chest_pos_z", blockPos.getZ());

        pContext.getItemInHand().setTag(chestLocation);
        messageChestSet((ServerPlayer) pContext.getPlayer());
    }

    private boolean chestSet(@NotNull ItemStack itemStack) {
        return itemStack.getTag() != null && itemStack.getTag().contains("chest_pos_x") && itemStack.getTag().contains("chest_pos_y") && itemStack.getTag().contains("chest_pos_z");
    }

    private void openChest(Level level, ServerPlayer player, ItemStack itemStack) {
        if (chestSet(itemStack)) {
            if (isChest(level, getPos(itemStack))) {
                ((ChestBlockEntity) level.getBlockEntity(getPos(itemStack))).startOpen(player);
                player.openMenu((ChestBlockEntity) level.getBlockEntity(getPos(itemStack)));
            } else {
                unsetChest(itemStack);
                messageChestMissing(player);
            }
        }
    }

    private void messageChestSet(@NotNull ServerPlayer player) {
        player.sendSystemMessage(Component.translatable(String.format("message.%s.%s", MODID, "chest_set")).withStyle(ChatFormatting.DARK_GREEN));
    }

    private void messageChestMissing(@NotNull ServerPlayer player) {
        player.sendSystemMessage(Component.translatable(String.format("message.%s.%s", MODID, "chest_missing")).withStyle(ChatFormatting.DARK_RED));
    }

    private void unsetChest(ItemStack itemStack) {
        CompoundTag cTag = itemStack.getTag();
        assert cTag != null;
        cTag.remove("chest_pos_x");
        cTag.remove("chest_pos_y");
        cTag.remove("chest_pos_z");
    }

    private BlockPos getPos(@NotNull ItemStack itemStack) {
        CompoundTag cTag = itemStack.getTag();
        assert cTag != null;
        return new BlockPos(cTag.getInt("chest_pos_x"), cTag.getInt("chest_pos_y"), cTag.getInt("chest_pos_z"));
    }

    @Override
    public void appendHoverText(@NotNull ItemStack pStack, @Nullable Level pLevel, @NotNull List<Component> pTooltipComponents, @NotNull TooltipFlag pIsAdvanced) {
        String key = String.format("tooltip.%s.%s", MODID, this);
        MutableComponent toolTip = Component.translatable(key);
        if (!toolTip.getString().equals(key) && chestSet(pStack)) {
            BlockPos pos = getPos(pStack);
            pTooltipComponents.add(toolTip.withStyle(ChatFormatting.DARK_BLUE).append(Component.literal(pos.getX() + "," + pos.getY() + "," + pos.getZ()).withStyle(ChatFormatting.WHITE)));
            super.appendHoverText(pStack, pLevel, pTooltipComponents, pIsAdvanced);
        }
    }
}
